package com.vcarecity.test.service.elastic;

import com.google.inject.Inject;
import com.vcarecity.elastic.util.AgencyPathUtil;
import com.vcarecity.test.service.QueryDataService;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.TermQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * @author Kerry on 19/12/10
 */


public class QueryDataServiceImpl implements QueryDataService {

    private final RestHighLevelClient restHighLevelClient;

    @Inject
    public QueryDataServiceImpl(RestHighLevelClient restHighLevelClient) {
        this.restHighLevelClient = restHighLevelClient;
    }


    @Override
    public void queryByCondition(String agencyPath, String filter) throws IOException {
        List<Integer> agencyIds = AgencyPathUtil.splitAgencyId(agencyPath);
        BoolQueryBuilder boolQueryBuilder = agencyCondition(agencyIds);

        String index = "unit-agency";


        SearchRequest searchRequest = new SearchRequest();
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        final BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery().must(boolQueryBuilder);
        System.out.println(queryBuilder);
        searchSourceBuilder.query(queryBuilder);
        searchSourceBuilder.fetchSource(new String[]{"unitId", "agencyPath"}, new String[]{});
        searchSourceBuilder.size(1000);

        searchRequest.source(searchSourceBuilder);
        searchRequest.indices(index);
        SearchResponse searchResponse = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);

        final SearchHits hits = searchResponse.getHits();
        int count = 0;
        long time = System.currentTimeMillis();
        for (SearchHit hit : hits.getHits()) {
            System.out.println((count++) + " - " + hit.getSourceAsString());
        }
        System.out.println((System.currentTimeMillis() - time) + " /ms");
    }


    private BoolQueryBuilder agencyCondition(List<Integer> agencyIds) {

        if (agencyIds == null || agencyIds.size() == 0) {
            return null;
        }
        final BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        final Map<String, Integer> map = AgencyPathUtil.agencyPathQueryMap(agencyIds);

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            TermQueryBuilder termQueryBuilder = QueryBuilders.termQuery(entry.getKey(), entry.getValue());
            boolQueryBuilder.must(termQueryBuilder);
        }
        return boolQueryBuilder;
    }


}
