package com.vcarecity.test.typehandler;

import com.vcarecity.test.core.ReflectionUtil;

import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Kerry on 19/12/10
 */

public class AutoTypeHandler implements TypeHandler {

    @Override
    public <T> List<T> resultHandler(ResultSet resultSet, Class<T> cls) throws SQLException, IllegalAccessException, InstantiationException {

        List<TypeParam> tps = TYPE_CACHE.get(cls);
        if (tps == null) {
            List<TypeParam> list = ReflectionUtil.reflectOrmClass(cls);
            TYPE_CACHE.put(cls, list);
            tps = TYPE_CACHE.get(cls);
        }

        List<T> list = new ArrayList<>();

        while (resultSet.next()) {
            T t = cls.newInstance();
            for (TypeParam tp : tps) {
                Object value = resultSet.getObject(tp.getColumnName());
                if (Modifier.isPrivate(tp.getField().getModifiers())) {
                    tp.getField().setAccessible(true);
                }
                if (value instanceof BigDecimal) {
                    Class<?> declaringClass = tp.getField().getType();
                    if (declaringClass == BigDecimal.class) {
                        tp.getField().set(t, value);
                    } else if (declaringClass == Long.class) {
                        tp.getField().set(t, ((BigDecimal) value).longValue());
                    } else if (declaringClass == Integer.class) {
                        tp.getField().set(t, ((BigDecimal) value).intValue());
                    } else {
                        tp.getField().set(t, value);
                    }
                } else if (value instanceof Timestamp) {
                    Class<?> declaringClass = tp.getField().getType();
                    if (declaringClass == LocalDateTime.class) {
                        tp.getField().set(t, ((Timestamp) value).toLocalDateTime());
                    } else if (declaringClass == java.util.Date.class) {
                        tp.getField().set(t, java.util.Date.from(((Timestamp) value).toInstant()));
                    } else {
                        tp.getField().set(t, value);
                    }
                } else if (value instanceof Date) {
                    Class<?> declaringClass = tp.getField().getType();
                    if (declaringClass == LocalDate.class) {
                        tp.getField().set(t, ((Date) value).toLocalDate());
                    } else if (declaringClass == java.util.Date.class) {
                        tp.getField().set(t, java.util.Date.from(((Date) value).toInstant()));
                    } else {
                        tp.getField().set(t, value);
                    }
                } else {
                    tp.getField().set(t, value);
                }

            }
            list.add(t);
        }

        return list;
    }
}
