package com.vcarecity.elastic.publish.service.impl;

import com.vcarecity.elastic.publish.entity.UnitAgencyMergeEntity;
import com.vcarecity.elastic.publish.service.UpdateElasticService;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.reindex.BulkByScrollResponse;
import org.elasticsearch.index.reindex.DeleteByQueryRequest;
import org.elasticsearch.index.reindex.UpdateByQueryRequest;
import org.elasticsearch.script.Script;
import org.elasticsearch.script.ScriptType;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Kerry on 12/17/19
 */

@Service
public class UpdateElasticServiceImpl implements UpdateElasticService {

    public static final String INDEX = "unit-agency";

    private final RestHighLevelClient restHighLevelClient;

    public UpdateElasticServiceImpl(RestHighLevelClient restHighLevelClient) {
        this.restHighLevelClient = restHighLevelClient;
    }


    @Override
    public void updateTest() throws IOException {

        Map<String, Object> params = new HashMap<>();
        params.put("dtuNo", "000000000001");
        params.put("puserCode", "/0/000000000001/1/12345678/");


        UpdateByQueryRequest request = new UpdateByQueryRequest("unit-agency");
        request.setQuery(QueryBuilders.termQuery("unitId", 12345678));
        request.setMaxDocs(1);
        request.setBatchSize(1);
        request.setScript(new Script(ScriptType.INLINE, "painless", getPainlessCode(params), params));
        request.setRefresh(true);


        BulkByScrollResponse response = restHighLevelClient.updateByQuery(request, RequestOptions.DEFAULT);

        System.out.println(response.getStatus());

    }


    private String getPainlessCode(Map<String, Object> params) {
        String[] items = new String[params.size()];
        int index = 0;
        for (String key : params.keySet()) {
            items[index++] = "ctx._source." + key + "=params." + key;
        }
        return String.join(";", items);
    }


    @Override
    public void updateUnit(List<UnitAgencyMergeEntity> list) throws IOException {
        List<Long> deleteId = new ArrayList<>();
        for (UnitAgencyMergeEntity uae : list) {
            if (uae.getIsDeleted() == 1) {
                deleteId.add(uae.getUnitId());
            }
        }
        deleteById("unitId", deleteId);

    }

    @Override
    public void updateAgency(List<UnitAgencyMergeEntity> list) throws IOException {
        List<UnitAgencyMergeEntity> updateList = new ArrayList<>();
        List<Long> deleteId = new ArrayList<>();
        for (UnitAgencyMergeEntity uae : list) {
            if (uae.getIsDeleted() == 1) {
                deleteId.add(uae.getAgencyId());
            } else if (uae.getIsDeleted() == 0) {
                updateList.add(uae);
            }
        }
        deleteById("agencyId", deleteId);
        list.clear();

    }


    private void insertData(List<UnitAgencyMergeEntity> list) {

    }

    private void deleteById(String key, List<Long> deleteId) throws IOException {
        if (deleteId == null || deleteId.isEmpty()) {
            return;
        }
        for (Long id : deleteId) {
            DeleteByQueryRequest request = new DeleteByQueryRequest(INDEX);
            request.setQuery(QueryBuilders.termQuery(key, id));
            restHighLevelClient.deleteByQuery(request, RequestOptions.DEFAULT);
        }
    }

}
